package cmd

import (
	"fmt"
	"time"

	"github.com/urfave/cli/v2"

	"codeberg.org/codeberg/pages/server/database"
)

var Certs = &cli.Command{
	Name:  "certs",
	Usage: "manage certs manually",
	Subcommands: []*cli.Command{
		{
			Name:   "list",
			Usage:  "list all certificates in the database",
			Action: listCerts,
		},
		{
			Name:   "remove",
			Usage:  "remove a certificate from the database",
			Action: removeCert,
		},
	},
	Flags: []cli.Flag{
		// Cert Storage
		// TODO: remove in next version
		&cli.StringFlag{
			// DEPRICATED
			Name:    "db-pogreb",
			Value:   "key-database.pogreb",
			EnvVars: []string{"DB_POGREB"},
		},
		&cli.StringFlag{
			Name:    "db-type",
			Value:   "sqlite",
			EnvVars: []string{"DB_TYPE"},
		},
		&cli.StringFlag{
			Name:    "db-conn",
			Value:   "", // TODO: "certs.sqlite", in next version
			EnvVars: []string{"DB_CONN"},
		},
	},
}

func listCerts(ctx *cli.Context) error {
	keyDatabase, err := database.NewPogreb(ctx.String("db-pogreb"))
	if err != nil {
		return fmt.Errorf("could not create database: %v", err)
	}

	items, err := keyDatabase.Items(0, 0)
	if err != nil {
		return err
	}

	fmt.Printf("Name\tDomain\tValidTill\n\n")
	for _, cert := range items {
		if cert.Name[0] == '.' {
			cert.Name = "*" + cert.Name
		}
		fmt.Printf("%s\t%s\t%s\n",
			cert.Name,
			cert.Domain,
			time.Unix(cert.ValidTill, 0).Format(time.RFC3339))
	}
	return nil
}

func removeCert(ctx *cli.Context) error {
	if ctx.Args().Len() < 1 {
		return fmt.Errorf("'certs remove' requires at least one domain as an argument")
	}

	domains := ctx.Args().Slice()

	keyDatabase, err := database.NewPogreb(ctx.String("db-pogreb"))
	if err != nil {
		return fmt.Errorf("could not create database: %v", err)
	}

	for _, domain := range domains {
		fmt.Printf("Removing domain %s from the database...\n", domain)
		if err := keyDatabase.Delete(domain); err != nil {
			return err
		}
	}
	if err := keyDatabase.Close(); err != nil {
		return err
	}
	return nil
}
